@@ -5,9 +5,9 @@
 ;; Author: Steve Purcell <steve@sanityinc.com>
 ;; Keywords: processes, tools
 ;; Homepage: https://github.com/purcell/envrc
-;; Package-Requires: ((emacs "26.1") (inheritenv "0.1"))
-;; Package-Version: 0.12
-;; Package-Revision: 2316e004c157
+;; Package-Requires: ((emacs "27.1") (inheritenv "0.1") (seq "2.24"))
+;; Package-Version: 20250917.915
+;; Package-Revision: de1ae6e53876
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -46,7 +46,6 @@
 ;;; Code:
 
 ;; TODO: special handling for DIRENV_* vars? exclude them? use them to safely reload more aggressively?
-;; TODO: special handling for remote files
 ;; TODO: handle nil default-directory (rarely happens, but is possible)
 ;; TODO: limit size of *direnv* buffer
 ;; TODO: special handling of compilation-environment?
@@ -65,6 +64,7 @@
 (require 'cl-lib)
 (require 'diff-mode) ; for its faces
 (require 'inheritenv)
+(eval-when-compile (require 'tramp))
 
 ;;; Custom vars and minor modes
 
@@ -128,6 +128,14 @@ To access `envrc-command-map' from this map, give it a prefix keybinding,
 e.g. (define-key envrc-mode-map (kbd \"C-c e\") \\='envrc-command-map)"
   :type '(restricted-sexp :match-alternatives (keymapp)))
 
+(defcustom envrc-remote nil
+  "Whether or not to enable direnv over TRAMP."
+  :type 'boolean)
+
+(defcustom envrc-supported-tramp-methods '("ssh")
+  "Tramp connection methods that are supported by envrc."
+  :type '(repeat string))
+
 ;;;###autoload
 (define-minor-mode envrc-mode
   "A local minor mode in which env vars are set by direnv."
@@ -144,9 +152,17 @@ e.g. (define-key envrc-mode-map (kbd \"C-c e\") \\='envrc-command-map)"
 
 ;;;###autoload
 (define-globalized-minor-mode envrc-global-mode envrc-mode
-  (lambda () (when (and (not (minibufferp)) (not (file-remote-p default-directory))
-                        (executable-find envrc-direnv-executable))
-               (envrc-mode 1))))
+  (lambda ()
+    (when
+        (cond
+         ((minibufferp) nil)
+         ((file-remote-p default-directory)
+          (and envrc-remote
+               (seq-contains-p
+                envrc-supported-tramp-methods
+                (with-parsed-tramp-file-name default-directory vec vec-method))))
+         (t (executable-find envrc-direnv-executable)))
+      (envrc-mode 1))))
 
 (defface envrc-mode-line-on-face '((t :inherit success))
   "Face used in mode line to indicate that direnv is in effect.")
@@ -160,7 +176,7 @@ e.g. (define-key envrc-mode-map (kbd \"C-c e\") \\='envrc-command-map)"
 ;;; Global state
 
 (defvar envrc--cache (make-hash-table :test 'equal :size 10)
-  "Known envrc directorie and their direnv results.
+  "Known envrc directories and their direnv results.
 The values are as produced by `envrc--export'.")
 
 ;;; Local state
@@ -169,6 +185,11 @@ The values are as produced by `envrc--export'.")
   "Symbol indicating state of the current buffer's direnv.
 One of \\='(none on error).")
 
+(defvar-local envrc--remote-path nil
+  "Buffer local variable for remote path.
+If set, this will override `tramp-remote-path' via connection
+local variables.")
+
 ;;; Internals
 
 (defun envrc--lighter ()
@@ -202,23 +223,23 @@ called `cd'"
   "Get a hash key for the result of invoking direnv in ENV-DIR with PROCESS-ENV.
 PROCESS-ENV should be the environment in which direnv was run,
 since its output can vary according to its initial environment."
-  (mapconcat 'identity (cons env-dir process-env) "\0"))
+  (string-join (cons env-dir process-env) "\0"))
 
 (defun envrc--update ()
   "Update the current buffer's environment if it is managed by direnv.
 All envrc.el-managed buffers with this env will have their
 environments updated."
-  (let ((env-dir (envrc--find-env-dir)))
-    (let ((result
-           (if env-dir
-               (let ((cache-key (envrc--cache-key env-dir (default-value 'process-environment))))
-                 (pcase (gethash cache-key envrc--cache 'missing)
-                   (`missing (let ((calculated (envrc--export env-dir)))
-                               (puthash cache-key calculated envrc--cache)
-                               calculated))
-                   (cached cached)))
-             'none)))
-      (envrc--apply (current-buffer) result))))
+  (let* ((env-dir (envrc--find-env-dir))
+         (result
+          (if env-dir
+              (let ((cache-key (envrc--cache-key env-dir (default-value 'process-environment))))
+                (pcase (gethash cache-key envrc--cache 'missing)
+                  (`missing (let ((calculated (envrc--export env-dir)))
+                              (puthash cache-key calculated envrc--cache)
+                              calculated))
+                  (cached cached)))
+            'none)))
+    (envrc--apply (current-buffer) result)))
 
 (defmacro envrc--at-end-of-special-buffer (name &rest body)
   "At the end of `special-mode' buffer NAME, execute BODY.
@@ -244,11 +265,11 @@ MSG and ARGS are as for that function."
   "Create a summary string for ITEMS."
   (if items
       (cl-loop for (name . val) in items
-               if (not (string-prefix-p "DIRENV_" name))
+               with process-environment = (default-value 'process-environment)
+               unless (string-prefix-p "DIRENV_" name)
                collect (cons name
                              (if val
-                                 (if (let ((process-environment (default-value 'process-environment)))
-                                       (getenv name))
+                                 (if (getenv name)
                                      '("~" diff-changed)
                                    '("+" diff-added))
                                '("-" diff-removed)))
@@ -303,9 +324,10 @@ variable names and values."
               (envrc--at-end-of-special-buffer "*envrc*"
                 (insert "──── " (format-time-string "%Y-%m-%d %H:%M:%S") " ──── " env-dir " ────\n\n")
                 (let ((initial-pos (point)))
-                  (insert-file-contents (let (ansi-color-context)
-                                          (ansi-color-apply stderr-file)))
+                  (insert-file-contents stderr-file)
                   (goto-char (point-max))
+                  (let (ansi-color-context)
+                    (ansi-color-apply-on-region initial-pos (point)))
                   (add-face-text-property initial-pos (point) (if (eq 0 exit-code) 'success 'error)))
                 (insert "\n\n")
                 (when (and (numberp exit-code) (/= 0 exit-code))
@@ -313,8 +335,9 @@ variable names and values."
       (delete-file stderr-file))
     result))
 
-;; Forward declaration for the byte compiler
+;; Forward declarations for the byte compiler
 (defvar eshell-path-env)
+(defvar Info-directory-list)
 
 (defun envrc--merged-environment (process-env pairs)
   "Make a `process-environment' value that merges PROCESS-ENV with PAIRS.
@@ -335,7 +358,7 @@ also appear in PAIRS."
   (with-current-buffer buf
     (kill-local-variable 'exec-path)
     (kill-local-variable 'process-environment)
-    (kill-local-variable 'info-directory-list)
+    (kill-local-variable 'Info-directory-list)
     (when (derived-mode-p 'eshell-mode)
       (if (fboundp 'eshell-set-path)
           (eshell-set-path (butlast exec-path))
@@ -351,15 +374,31 @@ also appear in PAIRS."
           (envrc--clear buf)
           (envrc--debug "[%s] reset environment to default" buf))
       (envrc--debug "[%s] applied merged environment" buf)
-      (setq-local process-environment (envrc--merged-environment (default-value 'process-environment) result))
-      (let ((path (getenv "PATH"))) ;; Get PATH from the merged environment: direnv may not have changed it
-        (setq-local exec-path (parse-colon-path path))
+      (let* ((remote (when-let* ((fn (buffer-file-name buf)))
+                       (file-remote-p fn)))
+             (env (envrc--merged-environment
+                   (default-value (if remote
+                                      'tramp-remote-process-environment
+                                    'process-environment))
+                   result))
+             (path (getenv-internal "PATH" env))
+             (parsed-path (parse-colon-path path)))
+        (if remote
+            (setq-local tramp-remote-process-environment env)
+          (setq-local process-environment env))
+        ;; Get PATH from the merged environment: direnv may not have changed it
+        (if remote
+            (setq-local envrc--remote-path parsed-path)
+          (setq-local exec-path parsed-path))
         (when (derived-mode-p 'eshell-mode)
           (if (fboundp 'eshell-set-path)
               (eshell-set-path path)
-            (setq-local eshell-path-env path))))
-      ;; Force info.el to parse INFOPATH again in case direnv modified it
-      (when (getenv "INFOPATH") (setq-local Info-directory-list nil)))))
+            (setq-local eshell-path-env path)))
+        (when-let* ((info-path (getenv-internal "INFOPATH" env)))
+          (setq-local Info-directory-list
+                      (append (seq-filter #'identity (parse-colon-path info-path))
+                              (when (boundp 'Info-directory-list)
+                                (default-value 'Info-directory-list)))))))))
 
 (defun envrc--update-env (env-dir)
   "Refresh the state of the direnv in ENV-DIR and apply in all relevant buffers."
@@ -398,40 +437,35 @@ In particular, we ensure the default variable `exec-path' and
 ARGS is as for `call-process'."
   (let ((exec-path (default-value 'exec-path))
         (process-environment (default-value 'process-environment)))
-    (apply 'call-process args)))
+    (apply 'process-file args)))
+
+(defun envrc--run-direnv (verb)
+  "Run direnv command named by VERB, then refresh current env."
+  (envrc--with-required-current-env env-dir
+    (let* ((outbuf (get-buffer-create (format "*envrc-%s*" verb)))
+           (default-directory env-dir)
+           (exit-code (envrc--call-process-with-global-env envrc-direnv-executable nil outbuf nil verb)))
+      (if (zerop exit-code)
+          (progn
+            (envrc--update-env env-dir)
+            (kill-buffer outbuf))
+        (display-buffer outbuf)
+        (user-error "Error running direnv %s" verb)))))
 
 (defun envrc-reload ()
   "Reload the current env."
   (interactive)
-  (envrc--with-required-current-env env-dir
-    (let* ((default-directory env-dir)
-           (exit-code (envrc--call-process-with-global-env envrc-direnv-executable nil (get-buffer-create "*envrc-reload*") nil "reload")))
-      (if (zerop exit-code)
-          (envrc--update-env env-dir)
-        (display-buffer "*envrc-reload*")
-        (user-error "Error running direnv reload")))))
+  (envrc--run-direnv "reload"))
 
 (defun envrc-allow ()
   "Run \"direnv allow\" in the current env."
   (interactive)
-  (envrc--with-required-current-env env-dir
-    (let* ((default-directory env-dir)
-           (exit-code (envrc--call-process-with-global-env envrc-direnv-executable nil (get-buffer-create "*envrc-allow*") nil "allow")))
-      (if (zerop exit-code)
-          (envrc--update-env env-dir)
-        (display-buffer "*envrc-allow*")
-        (user-error "Error running direnv allow")))))
+  (envrc--run-direnv "allow"))
 
 (defun envrc-deny ()
   "Run \"direnv deny\" in the current env."
   (interactive)
-  (envrc--with-required-current-env env-dir
-    (let* ((default-directory env-dir)
-           (exit-code (envrc--call-process-with-global-env envrc-direnv-executable nil (get-buffer-create "*envrc-deny*") nil "deny")))
-      (if (zerop exit-code)
-          (envrc--update-env env-dir)
-        (display-buffer "*envrc-deny*")
-        (user-error "Error running direnv deny")))))
+  (envrc--run-direnv "deny"))
 
 (defun envrc-reload-all ()
   "Reload direnvs for all buffers.
@@ -446,7 +480,7 @@ This can be useful if a .envrc has been deleted."
 (defun envrc-show-log ()
   "Open envrc log buffer."
   (interactive)
-  (if-let ((buffer (get-buffer "*envrc*")))
+  (if-let* ((buffer (get-buffer "*envrc*")))
       (pop-to-buffer buffer)
     (message "Envrc log buffer does not exist")))
 
@@ -461,13 +495,36 @@ in a temp buffer.  ARGS is as for ORIG."
       (inheritenv (apply orig args))
     (apply orig args)))
 
-(advice-add 'shell-command-to-string :around #'envrc-propagate-environment)
-(advice-add 'async-shell-command :around #'envrc-propagate-environment)
+(defun envrc-propagate-tramp-environment (buf)
+  "Advice function to propagate `tramp-remote-path' and
+`tramp-remote-process-environment' from buffer local values."
+  (when envrc-mode
+    (let ((cur-path envrc--remote-path)
+          (cur-env tramp-remote-process-environment))
+      (with-current-buffer buf
+        (setq-local tramp-remote-process-environment cur-env)
+        (setq-local envrc--remote-path cur-path))))
+  buf)
+
+(defun envrc-get-remote-path (fn vec)
+  "Advice function to wrap FN (`tramp-get-remote-path'),
+with its argument VEC.
+Shortcuts tramp caching direnv sets the variable `exec-path'."
+  (with-current-buffer (tramp-get-connection-buffer vec)
+    (or envrc--remote-path
+        (apply fn vec nil))))
+
+(advice-add 'shell-command :around #'envrc-propagate-environment)
 (advice-add 'org-babel-eval :around #'envrc-propagate-environment)
+(advice-add 'org-export-file :around #'envrc-propagate-environment)
+(advice-add 'tramp-get-connection-buffer :filter-return #'envrc-propagate-tramp-environment)
+(advice-add 'tramp-get-remote-path :around #'envrc-get-remote-path)
 
 
 ;;; Major mode for .envrc files
 
+;; Generate direnv keywords with:
+;;     $ rg "Usage:\s+([^_]\w+)" DIRENV_SRC/stdlib.sh -Nor '"$1"' | sort | uniq
 (defvar envrc-file-extra-keywords
   '("MANPATH_add" "PATH_add" "PATH_rm" "direnv_apply_dump" "direnv_layout_dir"
     "direnv_load" "direnv_version" "dotenv" "dotenv_if_exists"
@@ -475,7 +532,8 @@ in a temp buffer.  ARGS is as for ORIG."
     "layout" "load_prefix" "log_error" "log_status" "on_git_branch" "path_add"
     "path_rm" "rvm" "semver_search" "source_env" "source_env_if_exists"
     "source_up" "source_up_if_exists" "source_url" "strict_env" "unstrict_env"
-    "use" "use_guix" "use_flake" "use_nix" "user_rel_path" "watch_dir" "watch_file")
+    "use" "use_flake" "use_flox" "use_guix" "use_nix" "use_vim" "user_rel_path"
+    "watch_dir" "watch_file")
   "Useful direnv keywords to be highlighted.")
 
 (declare-function sh-set-shell "sh-script")
